﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Collections;
using System.Diagnostics;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Collections.ObjectModel;


namespace KeyMaker
{
    class Controller : INotifyPropertyChanged
    {
        private int mWidth;
        private int mHeight;
        private int mDPI;

        private int mSizeMain;
        private int mSizeSub;

        private int mYMain;
        private int mYSub;

        private bool mPreview;
        private bool mAntiAliasing;

        private Color mColorMain;
        private Color mColorSub;
        private FontFamily mFontFamily;

        public int Width
        {
            get{return this.mWidth;}
            set{this.mWidth = value;this.Notify("Width");}        
        }

        public int Height
        {
            get { return this.mHeight; }
            set { this.mHeight = value; this.Notify("Height"); }        
        }

        public int DPI
        {
            get { return this.mDPI; }
            set { this.mDPI = value; this.Notify("DPI"); }
        }

        public int SizeMain
        {
            get { return this.mSizeMain; }
            set { this.mSizeMain = value; this.Notify("SizeMain"); }
        }

        public int SizeSub
        {
            get { return this.mSizeSub; }
            set { this.mSizeSub = value; this.Notify("SizeSub"); }
        }

        public string ColorMain
        {
            get {return System.Drawing.ColorTranslator.ToHtml(this.mColorMain);}
            set { this.mColorMain = System.Drawing.ColorTranslator.FromHtml(value); this.Notify("ColorMain"); }
        }

        public string ColorSub
        {
            get { return System.Drawing.ColorTranslator.ToHtml(this.mColorSub); }
            set { this.mColorSub = System.Drawing.ColorTranslator.FromHtml(value); this.Notify("ColorSub"); }
        }

        public int YMain
        {
            get { return this.mYMain; }
            set { this.mYMain = value; this.Notify("YMain"); }
        }
        public int YSub
        {
            get { return this.mYSub; }
            set { this.mYSub = value; this.Notify("YSub"); }
        }

        public bool Preview
        {
            get { return this.mPreview; }
            set { this.mPreview = value; this.Notify("Preview"); }
        }

        public bool AntiAliasing
        {
            get { return this.mAntiAliasing; }
            set { this.mAntiAliasing = value; this.Notify("AntiAliasing"); }
        }

        public void SetFontFamily(FontFamily ff)
        {
            this.mFontFamily = ff;                        
        }

        public ObservableCollection<XmlKeyItem> xmlItems = new ObservableCollection<XmlKeyItem>();

        public Controller()
        {
            this.DPI = 96;
            this.Width = 26;
            this.Height = 50;
            this.SizeMain = 14;
            this.SizeSub = 9;
            string value = ConvertToString(Color.White);
            this.ColorMain = value;
            value = ConvertToString(Color.YellowGreen);
            this.ColorSub = value;
            this.YMain = 24;
            this.YSub = 7;
            this.Preview = true;
            
        }

        private string ConvertToString(Color c)
        {
            string value = String.Format("#{0:X}{1:X}{2:X}", c.R, c.G, c.B);
            return value;
        }
         
        public void LoadXML(string xmlFile)
        {
            XmlDocument xd = new XmlDocument();
            xd.Load(xmlFile);
            XmlNodeList xnl = xd.GetElementsByTagName("Key");
            IEnumerator ie = xnl.GetEnumerator();
            this.xmlItems.Clear();
            while (ie.MoveNext())
            {
                XmlElement xe = (XmlElement)ie.Current;
                string codevalue = xe.GetAttribute("android:codes");
                string[] codes = codevalue.Split(new char[] {','});
                int MainCharCode = int.Parse(codes[0]);
                string mainChar = ((char)MainCharCode).ToString();
                string SubChar = xe.GetAttribute("android:popupCharacters");
                SubChar = SubChar.Replace("_UVOZOVKY_","\"");
                XmlKeyItem item = new XmlKeyItem();
                item.PrimaryKeyCode = MainCharCode;
                item.PrimaryKeyValue = mainChar;
                item.PopupCharacters = SubChar;
                item.Ignore = !(item.PrimaryKeyCode > 32 && item.PrimaryKeyCode < 256);
                if (SubChar.Length > 0)
                {
                    item.SecondaryValue = SubChar[0].ToString();
                    if (item.SecondaryValue.Equals(@"\") && !Char.IsLetter(SubChar[1]))
                        item.SecondaryValue = SubChar[1].ToString();
                }
                else
                    item.SecondaryValue = string.Empty;
                this.xmlItems.Add(item);
            }
        }
        

        XmlKeyItem lastItem = null;
        public MemoryStream handlePreview(XmlKeyItem item, System.Drawing.FontFamily fontFamily, int typ)
        {
            this.lastItem = item;
            PictureMaker pm = this.GetPictureMaker(fontFamily,typ);
            string value = (typ == 0) ? item.SecondaryValue : item.PopupCharacters;
            pm.CreatePicture(item.PrimaryKeyValue, value);
            MemoryStream ms = pm.SaveImage();
            pm = null;
            return ms;
        }

        public void handleGenerujClick(string outFolder, System.Drawing.FontFamily fontFamily, bool qwerty, int typ)
        {
            PictureMaker pm = this.GetPictureMaker(fontFamily,typ);
            foreach (XmlKeyItem item in this.xmlItems)
            {
                if (!item.Ignore)
                {
                    string value = (typ == 0) ? item.SecondaryValue : item.PopupCharacters;
                    pm.CreatePicture(item.PrimaryKeyValue, value);
                    string outfile = string.Empty;
                    if (typ == 1)
                    {
                        outfile = String.Format(@"{0}\KEY_num_{1}.png", outFolder, item.PrimaryKeyCode);                                                                      
                       
                    }
                    else
                    {
                        outfile = String.Format(@"{0}\KEY_{1}.png", outFolder, item.PrimaryKeyCode);
                        if (item.PrimaryKeyValue.Equals("z", StringComparison.OrdinalIgnoreCase) || item.PrimaryKeyValue.Equals("y", StringComparison.OrdinalIgnoreCase))
                        {
                            string sub = (qwerty) ? "qwerty" : "qwertz";
                            outfile = String.Format(@"{0}\KEY_{1}_{2}.png", outFolder, item.PrimaryKeyCode, sub);
                        }
                    }
                    pm.SaveImage(outfile);                    
                }
            }
        }

        private PictureMaker GetPictureMaker(System.Drawing.FontFamily fontFamily, int typ)
        {
            PictureMaker pm = null;
            if (typ == 0)
                pm = new PictureMaker();
            else
                pm = new NumericPictureMaker();

            pm.setMainColor(this.mColorMain);
            pm.setSubColor(this.mColorSub);
            pm.width = this.Width;
            pm.height = this.Height;
            pm.AntiAliasing = this.AntiAliasing;
            if (fontFamily != null)
                pm.SetFont(fontFamily, this.mSizeMain, this.mSizeSub);
            else
            {
                if (this.mFontFamily == null)
                {
                    pm.SetMainSize(this.mSizeMain);
                    pm.SetSubSize(this.mSizeSub);
                }
                else
                    pm.SetFont(this.mFontFamily, this.mSizeMain, this.mSizeSub);
            }
            pm.YMain = this.YMain;
            pm.YSub = this.YSub;
            return pm;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void Notify(string Property)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(Property));                      
        }

        public void SaveState(string fileName, Dictionary<string,string> data)
        {
            StreamWriter sw = new StreamWriter(fileName);
            foreach (string key in data.Keys)
            {
                string dataValue = data[key];
                string line = string.Format("{0}={1}",key,dataValue);
                sw.WriteLine(line);
            }
            sw.Close();
        }

        public Dictionary<string,string> LoadState(string fileName)
        {
            StreamReader sr = new StreamReader(fileName);
            string line = null;
            Dictionary<string,string> data = new Dictionary<string,string>();
            while((line = sr.ReadLine()) != null)
            {
                string[] dataValues = line.Split(new char[] {'='});
                string key = dataValues[0];
                string value = dataValues[1];
                data.Add(key,value);
            }
            sr.Close();
            return data;
        }
    }
}
