﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using System.Collections;
using System.Diagnostics;
using System.IO;

namespace KeyMaker
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Controller controller;

        public MainWindow()
        {
            InitializeComponent();
            this.comboBox1.ItemsSource = System.Drawing.FontFamily.Families;                     
            this.comboBox1.SelectedValuePath = "Name";
            this.controller = (Controller)this.Resources["controller"];
        }

        private void btnOpen_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Microsoft.Win32.OpenFileDialog od = new Microsoft.Win32.OpenFileDialog();
                bool result = Convert.ToBoolean(od.ShowDialog());
                if(result)
                {
                    string file = od.FileName;
                    this.txtFile.Text = file;
                    controller.LoadXML(file);
                    this.dataGrid1.ItemsSource = controller.xmlItems;
                }
            }
            catch(Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message);
            }
        }

        private void btnGenerujObrazky_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                int type = 0;
                if (this.cmbTyp.SelectedItem != null)
                    type = Convert.ToInt32(((ComboBoxItem)this.cmbTyp.SelectedItem).ToolTip);

                bool qwerty = this.txtFile.Text.Contains("qwerty");
                string outFolder = this.txtOutputFolder.Text;
                System.Drawing.FontFamily ff = null;
                if (this.comboBox1.SelectedItem != null)
                    ff = (System.Drawing.FontFamily)this.comboBox1.SelectedItem;
                this.controller.handleGenerujClick(outFolder, ff, qwerty, type);
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message);
            }
            
        }

        public struct NumCode
        {            
            public string MainChar { get; set; }
            public string SubChars { get; set; }
            public string GetSubCharsSpaced()
            {
                string result = string.Empty;                                    
                if(this.SubChars.Length != 0)
                {
                    StringBuilder sb = new StringBuilder(10);
                    for (int i = 0; i < this.SubChars.Length;i++)
                    {
                        sb.Append(this.SubChars[i] + " ");
                    }
                    result = sb.ToString().Trim();
                }
                return result;
            }
        }


        public NumCode[] numcodes = new NumCode[] 
                                    {
                                        new NumCode{MainChar = "1",SubChars = ""},
                                        new NumCode{MainChar = "2",SubChars = "ABC"},
                                        new NumCode{MainChar = "3",SubChars = "DEF"},
                                        new NumCode{MainChar = "4",SubChars = "GHI"},
                                        new NumCode{MainChar = "5",SubChars = "JKL"},
                                        new NumCode{MainChar = "6",SubChars = "MNO"},
                                        new NumCode{MainChar = "7",SubChars = "PQRS"},
                                        new NumCode{MainChar = "8",SubChars = "TUV"},
                                        new NumCode{MainChar = "9",SubChars = "WXYZ"},
                                        new NumCode{MainChar = "*",SubChars = ""},
                                        new NumCode{MainChar = "0",SubChars = ""},
                                        new NumCode{MainChar = "#",SubChars = ""}                                        
                                    };

        private void btnGenerujNumeric_Click(object sender, RoutedEventArgs e)
        {
            string outFolder = this.txtOutputFolder.Text;
            int type = 0;
            if (this.cmbTyp.SelectedItem != null)
                type = Convert.ToInt32(((ComboBoxItem)this.cmbTyp.SelectedItem).ToolTip);

            bool qwerty = this.txtFile.Text.Contains("qwerty");
           
            System.Drawing.FontFamily ff = null;
            if (this.comboBox1.SelectedItem != null)
                ff = (System.Drawing.FontFamily)this.comboBox1.SelectedItem;
            this.controller.handleGenerujClick(outFolder, ff, qwerty, type);
        }

        XmlKeyItem lastItem = null;
        private void dataGrid1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                int type = 0;
                if (this.cmbTyp.SelectedItem != null)
                    type = Convert.ToInt32(((ComboBoxItem)this.cmbTyp.SelectedItem).ToolTip);

                this.image1.Source = null;
                XmlKeyItem xki = (XmlKeyItem)e.AddedItems[0];
                this.lastItem = xki;
                this.handlePreview(xki, type);
                
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message);
            }
        }

        private void handlePreview(XmlKeyItem xki,int typ)
        {
            System.Drawing.FontFamily ff = null;
            if (this.comboBox1.SelectedItem != null)
                ff = (System.Drawing.FontFamily)this.comboBox1.SelectedItem;
            MemoryStream ms = this.controller.handlePreview(xki, ff,typ);


            BitmapDecoder jbd = PngBitmapDecoder.Create(ms, BitmapCreateOptions.IgnoreImageCache, BitmapCacheOption.Default);
            this.image1.Source = jbd.Frames[0];
        }

        private void comboBox1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            System.Drawing.FontFamily ff = null;
            int type = 0;
            if (this.cmbTyp.SelectedItem != null)
                type = Convert.ToInt32(((ComboBoxItem)this.cmbTyp.SelectedItem).ToolTip);

            if (this.comboBox1.SelectedItem != null)
            {
                ff = (System.Drawing.FontFamily)this.comboBox1.SelectedItem;
                this.controller.SetFontFamily(ff);
            }

            try
            {
                if (this.lastItem != null)
                    this.handlePreview(lastItem, type);
            }
            catch { }
        }

        private void btnSaveState_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Microsoft.Win32.SaveFileDialog od = new Microsoft.Win32.SaveFileDialog();
                bool result = Convert.ToBoolean(od.ShowDialog());
                Dictionary<string, string> data = new Dictionary<string, string>();
                if (result)
                {
                    Grid g = this.gridStyle;
                    IEnumerator ie = g.Children.GetEnumerator();
                    while (ie.MoveNext())
                    {
                        TextBox tb = ie.Current as TextBox;
                        if (tb != null)
                        {
                            data.Add(tb.Name, tb.Text);
                        }

                        CheckBox cb = ie.Current as CheckBox;
                        if (cb != null)
                        {
                            data.Add(cb.Name, cb.IsChecked.ToString());
                        }
                    }
                    this.controller.SaveState(od.FileName,data);
                }
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message);
            }
        }

        private void btnLoadState_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Microsoft.Win32.OpenFileDialog od = new Microsoft.Win32.OpenFileDialog();
                bool result = Convert.ToBoolean(od.ShowDialog());
                if (result)
                {
                    string file = od.FileName;
                    Dictionary<string, string> data = this.controller.LoadState(file);
                    foreach (string key in data.Keys)
                    {
                        dynamic control = this.FindName(key);
                        if (control is TextBox)
                        {
                            TextBox tb = control as TextBox;
                            tb.Text = data[key];
                        }
                        if (control is CheckBox)
                            control.IsChecked = Convert.ToBoolean(data[key]);
                    }
                }
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message);
            }
        }
     
    }
}
